datablock staticShapeData(emptyShapeData) {
	shapeFile = "base/data/shapes/empty.dts";
};

function NodeGroup(%name, %file) {
	%obj = new ScriptGroup() {
		class = NodeGroup;
		file = %file;
	};

	if (isObject(MissionGroup)) {
		MissionGroup.add(%obj);
	}

	if (%name !$= "") {
		%obj.setName(%name);
	}

	return %obj;
}

function NodeGroup::onAdd(%this) {
	%this.debug = 0;

	if (%this.file !$= "") {
		if (isFile(%this.file)) {
			%this.import();
		}
	}
}

function NodeGroup::setDebug(%this, %debug) {
	%this.debug = %debug ? 1 : 0;

	for (%i = %this.getCount() - 1; %i >= 0; %i--) {
		%node = %this.getObject(%i);

		if (%this.debug) {
			%node.createDebug();
		}
		else {
			%node.deleteDebug();
		}
	}
}

function NodeGroup::import(%this, %file, %append) {
	if (%file $= "") {
		%file = %this.file;

		if (%file $= "") {
			error("ERROR: No file specified and no default file.");
			return;
		}
	}

	if (!isFile(%file)) {
		error("ERROR: File does not exist.");
		return;
	}

	%this.deleteAll();
	%nodes = 0;

	%fp = new FileObject();
	%fp.openForRead(%file);

	while (!%fp.isEOF()) {
		%line = %fp.readLine();

		if (%line $= "-" || %line $= "neighbors") {
			%neighbors = 1;
		}
		else if (%neighbors) {
			%node[getWord(%line, 0)].link(%node[getWord(%line, 1)]);
		}
		else {
			%node[%nodes] = %this.Node(getField(%line, 0), getField(%line, 1));
			%nodes++;
		}
	}

	%fp.close();
	%fp.delete();
}

function NodeGroup::export(%this, %file) {
	if (%file $= "") {
		%file = %this.file;

		if (%file $= "") {
			error("ERROR: No file specified and no default file.");
			return;
		}
	}

	if (!isWriteableFileName(%file)) {
		error("ERROR: File is not writeable.");
		return;
	}

	%count = %this.getCount();

	%fp = new FileObject();
	%fp.openForWrite(%file);

	if (!%count) {
		%fp.close();
		%fp.delete();

		return;
	}

	for (%i = 0; %i < %count; %i++) {
		%node = %this.getObject(%i);
		%index[%node] = %i;

		%fp.writeLine(%node.position TAB %node.flags);
	}

	%fp.writeLine("-");

	for (%i = 0; %i < %count; %i++) {
		%node = %this.getObject(%i);

		for (%j = 0; %j < %node.linkCount; %j++) {
			%fp.writeLine(%i SPC %index[%node.link[%j]]);
		}
	}

	%fp.close();
	%fp.delete();
}

function NodeGroup::findNearest(%this, %position, %radius, %occluders, %ignore, %flags) {
	initContainerRadiusSearch(%position, %radius, $TypeMasks::StaticShapeObjectType);

	while (isObject(%obj = containerSearchNext())) {
		%node = %obj.node;

		if (isObject(%node) && %node.getGroup() == %this && (%node.flags & %flags)) {
			if (%occluders !$= "") {
				%ray = containerRayCast(%position, %node.position, %occluders, %ignore);

				if (%ray !$= 0) {
					continue;
				}
			}

			return %node;
		}
	}

	return -1;
}

function NodeGroup::Node(%this, %position, %flags) {
	if (%flags $= "") {
		%flags = 0;
	}

	%obj = new ScriptObject() {
		class = Node;

		position = %position;
		flags = %flags;
	};

	%this.add(%obj);

	if (%this.debug) {
		%obj.createDebug();
	}

	return %obj;
}

function Node::onAdd(%this) {
	%this.linkCount = 0;
	%this.linkedBy = 0;

	%this.shape = new StaticShape() {
		position = %this.position;
		datablock = EmptyShapeData;

		node = %this;
	};
}

function Node::onRemove(%this) {
	if (isObject(%this.shape)) {
		%this.shape.delete();
	}

	for (%i = 0; %i < %this.linkedBy; %i++) {
		%node = %this.linkedBy[%i];
		%node.linkedTo[%this] = "";

		if (isObject(%node.debugLineTo[%this])) {
			%node.debugLineTo[%this].delete();
			%node.debugLineTo[%this] = "";
		}

		for (%j = %node.indexOfLink[%this]; %j < %node.linkCount; %j++) {
			%node.link[%j] = %node.link[%j + 1];
		}

		%node.linkCount--;
		%node.link[%node.linkCount] = "";
	}

	%this.deleteDebug();
}

function Node::link(%this, %node) {
	if (!isObject(%node) || %node.class !$= Node) {
		error("ERROR: Invalid node.");
		return;
	}

	if (%this.getGroup() != %node.getGroup()) {
		error("ERROR: Node is in another NodeGroup.");
		return;
	}

	if (%this == %node) {
		error("ERROR: Cannot connect a node to itself.");
		return;
	}

	if (%this.linkedTo[%node]) {
		return;
	}

	%this.linkedTo[%node] = 1;
	%this.indexOfLink[%node] = %this.links;

	%this.link[%this.linkCount] = %node;
	%this.linkCount++;

	%node.linkedBy[%node.linkedBy] = %this;
	%node.linkedBy++;

	if (%this.getGroup().debug) {
		%this.createDebug();
	}
}

function Node::unlink(%this, %node) {
	if (!isObject(%node) || %node.class !$= Node) {
		error("ERROR: Invalid node.");
		return;
	}

	if (%this.getGroup() != %node.getGroup()) {
		error("ERROR: Node is in another NodeGroup.");
		return;
	}

	%this = %this.getID();
	%node = %node.getID();

	if (!%this.linkedTo[%node]) {
		return;
	}

	if (isObject(%this.debugLineTo[%node])) {
		%this.debugLineTo[%node].delete();
		%this.debugLineTo[%node] = "";
	}

	if (isObject(%node.debugLineTo[%this])) {
		%node.debugLineTo[%this].delete();
		%node.debugLineTo[%this] = "";
	}

	for (%i = %this.indexOfLink[%node]; %i < %this.linkCount; %i++) {
		%this.link[%i] = %this.link[%i + 1];
	}

	%this.linkCount--;
	%this.link[%this.linkCount] = "";

	%this.linkedTo[%node] = "";
	%this.indexOfLink[%node] = "";

	%node.linkedBy[%node.linkedBy] = %this;
	%node.linkedBy++;
}

function Node::createDebug(%this) {
	%cubeColor = "0 0.75 0";
	%lineColor = vectorAdd(vectorScale(%cubeColor, 0.5), "0.5 0.5 0.5");

	%cubeScale = "0.5 0.5 0.4";
	%lineWidth = 0.005;

	if (!isObject(%this.debugCube)) {
		%this.debugCube = createCube(%this.position, %cubeScale, %cubeColor SPC 1);
	}

	for (%i = 0; %i < %this.linkCount; %i++) {
		%node = %this.link[%i];

		if (!isObject(%this.debugLineTo[%node])) {
			if (isObject(%node.debugLineTo[%this])) {
				%this.debugLineTo[%node] = %node.debugLineTo[%this];
			}
			else {
				%this.debugLineTo[%node] = createLine(%this.position, %node.position, %lineWidth, %lineColor SPC 1);

				if (%node.linkedTo[%this]) {
					%node.debugLineTo[%this] = %this.debugLineTo[%node];
				}
			}
		}
	}
}

function Node::deleteDebug(%this) {
	if (isObject(%this.debugCube)) {
		%this.debugCube.delete();
		%this.debugCube = "";
	}

	for (%i = 0; %i < %this.linkCount; %i++) {
		%node = %this.link[%i];

		if (isObject(%this.debugLineTo[%node])) {
			%this.debugLineTo[%node].delete();
			%this.debugLineTo[%node] = "";
		}
	}
}
