function AIManager(%name) {
	%obj = new ScriptGroup() {
		class = AIManager;
	};

	if (isObject(MissionGroup)) {
		MissionGroup.add(%obj);
	}

	if (%name !$= "") {
		%obj.setName(%name);
	}

	return %obj;
}

function AIManager::onAdd(%this) {
	%this.debug = 0;
	%this.index = 0;

	if (%this.tickQuota $= "") {
		%this.tickQuota = 8;
	}

	if (%this.tickInterval $= "") {
		%this.tickInterval = 100;
	}

	%this.tick = %this.schedule(0, tick);
}

function AIManager::setDebug(%this, %level) {
	%this.debug = mFloor(%level);

	if (%this.debug < 0) {
		%this.debug = 0;
	}
}

function AIManager::debug(%this, %obj, %line, %level, %error) {
	if (%level $= "") {
		%level = 1;
	}

	if (%this.debug < %level) {
		return;
	}

	if (!isObject(%obj) || !%this.isMember(%obj)) {
		return;
	}

	%string = "[" @ $Sim::Time @ "]" SPC %obj.getDataBlock().getName @ "(" @ %obj.getID() @ ") =>" SPC %line;

	if (%error) {
		error(%string);
	}
	else {
		warn(%string);
	}
}

function AIManager::tick(%this) {
	cancel(%this.tick);

	%count = %this.getCount();
	%index = %this.index;

	while (%count && %usage < %this.tickQuota) {
		%this.index = %this.index % %count;

		if (%this.index < %index) {
			break;
		}

		%obj = %this.getObject(%this.index);

		if (%obj.getState() $= "Dead") {
			%this.remove(%obj);
			MissionCleanup.add(%obj);

			%count--;
			continue;
		}

		%obj.updateAI();

		%usage++;
		%this.index++;
	}

	%this.tick = %this.schedule(%this.tickInterval, tick);
}

function AIManager::spawn(%this, %transform, %dataBlock) {
	if (%transform $= "") {
		%transform = pickSpawnPoint();
	}

	if (%dataBlock $= "") {
		%dataBlock = PlayerStandardArmor;
	}
	else if (!isObject(%dataBlock) || %datablock.getClassName() !$= PlayerData) {
		error("ERROR: Invalid datablock.");
		return -1;
	}

	%obj = new AIPlayer() {
		dataBlock = %dataBlock;
	};

	%this.add(%obj);
	%obj.setTransform(%transform);

	return %obj;
}
